{{
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
Cogject demonstration program - load all cogs through a common section of hub memory
and save program space

To run, first copy all .cog files to an SD card

Minimalist SD card loader program - disable RTC by setting the RTC constants to -1
Take FullDuplexSerial and remove the spin code and save as Cog_FullDuplexSerial.
Compile to a binary and rename with shorter filename serial.cog and copy to SD card
Take FullDuplexSerial and remove the cog code and save as Spin_FullDuplexSerial.
  Comment out cognew and cogstop and replace cognew with okay := rx_head so returns pointer to the parameter list
Add   Serial: "Spin_FullDuplexSerial.spin"    to the OBJ section
Temporarily add VT100 display driver to OBJ section for debugging
Add code to open the file and close it
Add string object
Open file and read file into CogArray
Load cog and run
Split Keyboard.spin into Cog_keyboard and Spin_keyboard
Compile cog_keyboard, save as .binary, rename to keyboard.cog, copy to sd card
Modify spin_keyboard so returns location of the parameter list
Modified load so discards first 24 bits, and now load is in sub object, not the main
Added VT100 code using the vga 80x40 cog loaded off the sd card



///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
}}

CON

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

  _SD_DO = 12
  _SD_CLK = 13
  _SD_DI = 14
  _SD_CS = 15
  _SD_WP = -1 ' -1 ifnot installed.
  _SD_CD = -1 ' -1 ifnot installed.

  _RTC_DAT = -1 ' -1 ifnot installed.
  _RTC_CLK = -1 ' -1 ifnot installed.


OBJ

  fat: "SD3.01_FATEngine.spin"
  rtc: "DS1307_RTCEngine.spin"
  VT100: "VT100_80_40"       ' VGA VT100 Terminal Driver 
  Serial: "Spin_FullDuplexSerial.spin"
  kb : "Spin_Keyboard.spin"
 
VAR
    long cogarray               ' recycle the cog space used by fatengine, which happens to be 494 longs

PUB Main | i
    
  fat.FATEngineStart(_SD_DO, _SD_CLK, _SD_DI, _SD_CS, _SD_WP, _SD_CD, _RTC_DAT, _RTC_CLK, -1)
  cogarray := fat.addressofcog                          ' get location of the fat cog array
  fat.mountPartition(0)                                 ' mount the sd card
  'ifnot(fat.partitionMounted)                          ' commented out - if the sd card isn't working, can't load a cog to be able to display this
  '  printstringcr(string("Card Not Mounted "))
  ReadCogFile(string("Serial.Cog"))
  serial.start(cogarray,31,30,0,38400)                ' start the cog at this baud rate

  ReadCogFile(string("Keyboard.Cog"))
  kb.start(cogarray,26,27)                              'start the keyboard
  
  ReadCogFile(string("VGA8040.Cog"))                    ' load terminal cog - used twice
  vt100.start(cogarray,16)                             ' start a simple VGA terminal
  
  printstringcr(string("Cogjects demonstration"))
  printstringcr(string("Hit a key"))
  i := kb.getkey
  printchar(i)
  crlf
  hexdump                                             'print out last cog to load
  printstringcr(string("Finished"))

PRI ReadCogFile(Filename)
  result := \fat.openFile(Filename, "R")                ' create file and open file
  result := \fat.readdata(cogarray,24)                 ' read first 24 bytes and discard
  result := \fat.readdata(cogarray,fat.filesize-24)    ' read the data into cogarray
  result := \fat.closefile                              ' close the file
  
PRI PrintStringCR(pstring)' sends pstring to the vga and serial port
     PrintString(pstring)                               ' print the string  
     crlf

PRI PrintString(pstring)' sends pstring to the vga and serial port with no CRLF
    vt100.str(pstring)                                 ' send to vga screen
    'serial.str(pstring)                                 ' send to serial port

PRI PrintChar(c) ' sends c to vga 
    vt100.out(c)                                        ' send to vga
    'serial.tx(c)                                        ' send to serial port
    
PRI CRLF
    PrintChar(13)                                        ' send to vga
    PrintChar(10)                                        ' send to vga

PRI Hexdump | i,j,k' print out all the cogject in hex, 32 bytes per line
    k :=0
    repeat i from 0 to 63 ' 2048 bytes is 64 rows x 32 bytes. 
      vt100.hex(k,4)
      vt100.out(32)
      repeat j from 0 to 31
        vt100.hex(cogarray[k],2)
        k +=1
      if k == 1024
        crlf
        printstringcr(string("Hit a key")) ' 2 pages
        kb.getkey    
      crlf
 
{{

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                  TERMS OF USE: MIT License
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the
// Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
// COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
}}